{% extends "registration/base.html" %}

{% load i18n %}


{% block title %}{% trans "Logged Out" %}{% endblock title %}

{% block page_main_title %}{% trans "Logged Out" %}{% endblock page_main_title %}

{% block content %}
    <p>{% trans "You have logged out." %}</p>
{% endblock content %}
