{% load crispy_forms_tags %}
{% load i18n %}
{% load url from future %}


{% block content %}
    <p>{% trans "Please log in using your email and password." %}</p>

    {% block form %}
        <form class="{% block form_class %}{% endblock form_class %}" action="{% url 'login' %}" method="post" >
            {% csrf_token %}

            {% block form_inner %}
                {{ form|crispy }}

                <a href="{% url 'password_reset' %}" class="forgotten-password">{% trans "Forgotten password?" %}</a>
            {% endblock form_inner %}

            {% block form_actions %}
                <div class="form-actions">
                    {% block form_actions_inner %}
                        <input type="submit" class="button" value="{% trans "Log in" %}">
                        <input type="hidden" name="next" value="{{ next }}">
                    {% endblock form_actions_inner %}
                </div>
            {% endblock form_actions %}
        </form>
    {% endblock form %}
{% endblock content %}
