{% extends "registration/base.html" %}

{% load i18n %}
{% load url from future %}


{% block title %}{% trans "Password Change Successful" %}{% endblock title %}

{% block page_main_title %}{% trans "Password Change Successful" %}{% endblock page_main_title %}

{% block content %}
    <p>{% trans "Your password was changed." %}</p>
{% endblock content %}
